// CI.c 

#include "com_hp_javaci_CI.h"

const int SUBSYS_CI = 2;  /* actual msgset #, not subsys # */

//**********************************************************
//********** JavaCI Intrinsic Call Support      ************
//**********************************************************

typedef union {
   int all; 
   struct {
      short info;
      short subsys;
   } p;
} mpestatus;


extern void HPCICOMMAND(int, void *, int *, jint *, jint);
extern void HPCIGETVAR(int, char*, int *, int, void*, int, void*, int, void*,
                                          int, void*, int, void*, int, void*); 
extern void HPERRMSG(int, int, int, short, int, void *, short *, int *);

/* HPCICOMMAND */
JNIEXPORT void JNICALL
Java_com_hp_javaci_CI_mpeHPCICOMMAND(JNIEnv    *env,
                               jobject    this,
                               jstring    j_cmdimage,
                               jintArray  j_parms) {

   // Get JNI fields into C types
   jboolean isCopy;
   char *cmd;
   jboolean isCopyParm;
   mpestatus  error; 
   jint       *parms   = (*env)->GetIntArrayElements(env, j_parms, &isCopyParm);
   jint       parmnum  = parms[1];
   jint       msglevel = parms[2];

   //cast away the const-ness
   (const char*) cmd   = (*env)->GetStringUTFChars(env, j_cmdimage, &isCopy);

   //fprintf(stderr,"HPCICOMMAND.buf=%s\n",cmd);
   
   error.all = parms[0];
   HPCICOMMAND(4, cmd, &error.all, &parmnum, msglevel);

   if ((error.p.subsys == 0) && (error.p.info != 0)) {
      error.p.subsys = SUBSYS_CI;
   }

   // return values which may have been changed by intrinsic
   parms[0] = error.all;
   parms[1] = parmnum;

   if (isCopy == JNI_TRUE) {
      (*env)->ReleaseStringUTFChars(env, j_cmdimage, cmd);
   }
   if (isCopyParm == JNI_TRUE) {
      (*env)->ReleaseIntArrayElements(env, j_parms, parms, 0);
   }
}

JNIEXPORT jint JNICALL
Java_com_hp_javaci_CI_HPCIGETVAR_1INT(JNIEnv *env, 
                                     jobject this,
                                     jstring j_varname) {
   jint varvalue;
   mpestatus status;
   jboolean isCopy;
   char *varname;
   //cast away the const-ness
   (const char*) varname = (*env)->GetStringUTFChars(env, j_varname, &isCopy);

   HPCIGETVAR(4, varname, &status.all, 1, &varvalue, 0,0, 0,0, 0,0, 0,0, 0,0);

   if (isCopy == JNI_TRUE) {
      (*env)->ReleaseStringUTFChars(env, j_varname, varname);
   }

   if (status.all != 0)
      return 0;
   else
      return varvalue;
}

JNIEXPORT jstring JNICALL
Java_com_hp_javaci_CI_HPCIGETVAR_1STRING(JNIEnv *env, 
                                        jobject this, 
                                        jstring j_varname) {

   char varvalue[1025];
   int actuallen;
   int maxlen = sizeof(varvalue);
   mpestatus status;
   jboolean isCopy;
   jstring j_varvalue;
   char *varname;
   //cast away the const-ness
   (const char*) varname = (*env)->GetStringUTFChars(env, j_varname, &isCopy);

   // Get the value of the string
   //fprintf(stderr,"nmHPCIGETVAR_STRING.varname=%s;%d\n",varname,strlen(varname));
   HPCIGETVAR(14, varname, &status.all,
       2, &varvalue, 
      10, &maxlen,
      11, &actuallen,
      0,0,
      0,0,
      0,0);

   if (isCopy == JNI_TRUE) {
      (*env)->ReleaseStringUTFChars(env, j_varname, varname);
   }

   if (status.all != 0) {
      int NOT_PROGRAMMATICALLY_AVAILABLE = -8169;
      
      if (status.p.info == NOT_PROGRAMMATICALLY_AVAILABLE) {
        fprintf(stderr,"A CI var within the value of %s is not programmatically available.\n",
          varname);
        fprintf(stderr,"Some of these are HPCIDEPTH, HPCMDNUM, HPREDOSIZE, ...\n");
        fprintf(stderr,"Do a SHOWVAR %s \n", varname);
      }
      return NULL;
   } else {
      varvalue[actuallen] = '\0';
      j_varvalue = (*env)->NewStringUTF(env, varvalue); 
      return j_varvalue;
   }
}

JNIEXPORT void JNICALL
Java_com_hp_javaci_CI_HPERRMSG(JNIEnv *env, jobject this, jint errno) {

   int displaycode = 5; /* display message to $STDLIST */
   int depth       = 1; /* last error */
   short dummy     = 0;
   short buflen    = 0;
   mpestatus status;
   char *buffer    = NULL;


   HPERRMSG(7, displaycode, depth, dummy, errno, buffer, &buflen, &status.all);

   if (status.all != 0) 
      printf("HPERRMSG failed, subsys %d error %d\n", 
                               status.p.subsys, status.p.info);
}

JNIEXPORT jint JNICALL
Java_com_hp_javaci_CI_system(JNIEnv *env, jobject this, jstring j_cmd) {
   
   jboolean isCopy;
   const char *command = (*env)->GetStringUTFChars(env, j_cmd, &isCopy);
   int      status;

   status = system(command);

   if (isCopy == JNI_TRUE) {
      (*env)->ReleaseStringUTFChars(env, j_cmd, command);
   }
   return status;
}
